@@ -1,9 +1,13 @@
 import configparser
 import os
+import uuid
 from dataclasses import dataclass
 from pathlib import Path
+
 from typing import Any, Optional
 
+from skyplane.exceptions import BadConfigException
+
 _FLAG_TYPES = {
     "autoconfirm": bool,
     "bbr": bool,
@@ -14,15 +18,26 @@ _FLAG_TYPES = {
     "multipart_enabled": bool,
     "multipart_min_threshold_mb": int,
     "multipart_min_size_mb": int,
+    "multipart_chunk_size_mb": int,
     "multipart_max_chunks": int,
     "num_connections": int,
     "max_instances": int,
     "autoshutdown_minutes": int,
+    "aws_use_spot_instances": bool,
+    "azure_use_spot_instances": bool,
+    "gcp_use_spot_instances": bool,
     "aws_instance_class": str,
     "azure_instance_class": str,
     "gcp_instance_class": str,
+    "aws_default_region": str,
+    "azure_default_region": str,
+    "gcp_default_region": str,
     "gcp_use_premium_network": bool,
+    "usage_stats": bool,
     "gcp_service_account_name": str,
+    "requester_pays": bool,
+    "native_cmd_enabled": bool,
+    "native_cmd_threshold_gb": int,
 }
 
 _DEFAULT_FLAGS = {
@@ -33,17 +48,28 @@ _DEFAULT_FLAGS = {
     "encrypt_socket_tls": False,
     "verify_checksums": True,
     "multipart_enabled": True,
-    "multipart_min_threshold_mb": 64,
-    "multipart_min_size_mb": 8,
+    "multipart_min_threshold_mb": 128,
+    "multipart_min_size_mb": 5,  # 5 MiB minimum size for multipart uploads
+    "multipart_chunk_size_mb": 64,
     "multipart_max_chunks": 9990,  # AWS limit is 10k chunks
     "num_connections": 32,
     "max_instances": 1,
     "autoshutdown_minutes": 15,
+    "aws_use_spot_instances": False,
+    "azure_use_spot_instances": False,
+    "gcp_use_spot_instances": False,
     "aws_instance_class": "m5.8xlarge",
     "azure_instance_class": "Standard_D32_v5",
     "gcp_instance_class": "n2-standard-32",
+    "aws_default_region": "us-east-1",
+    "azure_default_region": "eastus",
+    "gcp_default_region": "us-central1-a",
     "gcp_use_premium_network": True,
+    "usage_stats": True,
     "gcp_service_account_name": "skyplane-manual",
+    "requester_pays": False,
+    "native_cmd_enabled": True,
+    "native_cmd_threshold_gb": 2,
 }
 
 
@@ -64,22 +90,24 @@ class SkyplaneConfig:
     aws_enabled: bool
     azure_enabled: bool
     gcp_enabled: bool
+    anon_clientid: str
+    azure_principal_id: Optional[str] = None
     azure_subscription_id: Optional[str] = None
-    azure_tenant_id: Optional[str] = None
+    azure_resource_group: Optional[str] = None
+    azure_umi_name: Optional[str] = None
     azure_client_id: Optional[str] = None
-    azure_client_secret: Optional[str] = None
     gcp_project_id: Optional[str] = None
 
     @staticmethod
-    def default_config() -> "SkyplaneConfig":
-        return SkyplaneConfig(
-            aws_enabled=False,
-            azure_enabled=False,
-            gcp_enabled=False,
-        )
+    def generate_machine_id() -> str:
+        return uuid.UUID(int=uuid.getnode()).hex
 
-    @staticmethod
-    def load_config(path) -> "SkyplaneConfig":
+    @classmethod
+    def default_config(cls) -> "SkyplaneConfig":
+        return cls(aws_enabled=False, azure_enabled=False, gcp_enabled=False, anon_clientid=cls.generate_machine_id())
+
+    @classmethod
+    def load_config(cls, path) -> "SkyplaneConfig":
         """Load from a config file."""
         path = Path(path)
         config = configparser.ConfigParser()
@@ -87,6 +115,11 @@ class SkyplaneConfig:
             raise FileNotFoundError(f"Config file not found: {path}")
         config.read(path)
 
+        if "client" in config and "anon_clientid" in config["client"]:
+            anon_clientid = config.get("client", "anon_clientid")
+        else:
+            anon_clientid = cls.generate_machine_id()
+
         aws_enabled = False
         if "aws" in config:
             if "aws_enabled" in config["aws"]:
@@ -94,20 +127,23 @@ class SkyplaneConfig:
 
         azure_enabled = False
         azure_subscription_id = None
-        azure_tenant_id = None
         azure_client_id = None
-        azure_client_secret = None
+        azure_principal_id = None
+        azure_resource_group = None
+        azure_umi_name = None
         if "azure" in config:
             if "azure_enabled" in config["azure"]:
                 azure_enabled = config.getboolean("azure", "azure_enabled")
             if "subscription_id" in config["azure"]:
                 azure_subscription_id = config.get("azure", "subscription_id")
-            if "tenant_id" in config["azure"]:
-                azure_tenant_id = config.get("azure", "tenant_id")
             if "client_id" in config["azure"]:
                 azure_client_id = config.get("azure", "client_id")
-            if "client_secret" in config["azure"]:
-                azure_client_secret = config.get("azure", "client_secret")
+            if "principal_id" in config["azure"]:
+                azure_principal_id = config.get("azure", "principal_id")
+            if "resource_group" in config["azure"]:
+                azure_resource_group = config.get("azure", "resource_group")
+            if "umi_name" in config["azure"]:
+                azure_umi_name = config.get("azure", "umi_name")
 
         gcp_enabled = False
         gcp_project_id = None
@@ -117,14 +153,16 @@ class SkyplaneConfig:
             if "project_id" in config["gcp"]:
                 gcp_project_id = config.get("gcp", "project_id")
 
-        skyplane_config = SkyplaneConfig(
+        skyplane_config = cls(
             aws_enabled=aws_enabled,
             azure_enabled=azure_enabled,
             gcp_enabled=gcp_enabled,
+            anon_clientid=anon_clientid,
+            azure_principal_id=azure_principal_id,
             azure_subscription_id=azure_subscription_id,
-            azure_tenant_id=azure_tenant_id,
             azure_client_id=azure_client_id,
-            azure_client_secret=azure_client_secret,
+            azure_resource_group=azure_resource_group,
+            azure_umi_name=azure_umi_name,
             gcp_project_id=gcp_project_id,
         )
 
@@ -151,12 +189,14 @@ class SkyplaneConfig:
 
         if self.azure_subscription_id:
             config.set("azure", "subscription_id", self.azure_subscription_id)
-        if self.azure_tenant_id:
-            config.set("azure", "tenant_id", self.azure_tenant_id)
         if self.azure_client_id:
             config.set("azure", "client_id", self.azure_client_id)
-        if self.azure_client_secret:
-            config.set("azure", "client_secret", self.azure_client_secret)
+        if self.azure_principal_id:
+            config.set("azure", "principal_id", self.azure_principal_id)
+        if self.azure_resource_group:
+            config.set("azure", "resource_group", self.azure_resource_group)
+        if self.azure_umi_name:
+            config.set("azure", "umi_name", self.azure_umi_name)
 
         if "gcp" not in config:
             config.add_section("gcp")
@@ -165,6 +205,10 @@ class SkyplaneConfig:
         if self.gcp_project_id:
             config.set("gcp", "project_id", self.gcp_project_id)
 
+        if "client" not in config:
+            config.add_section("client")
+        config.set("client", "anon_clientid", self.anon_clientid)
+
         if "flags" not in config:
             config.add_section("flags")
 
@@ -194,3 +238,18 @@ class SkyplaneConfig:
             setattr(self, f"flag_{flag_name}", _map_type(value, _FLAG_TYPES.get(flag_name, str)))
         else:
             setattr(self, f"flag_{flag_name}", None)
+
+    def check_config(self):
+        valid_config = True
+        if self.anon_clientid is None:
+            valid_config = False
+        if self.azure_enabled and (self.azure_client_id is None or self.azure_principal_id is None or self.azure_subscription_id is None):
+            valid_config = False
+        if self.azure_resource_group is None:
+            self.azure_resource_group = "skyplane"
+        if self.azure_umi_name is None:
+            self.azure_umi_name = "skyplane_umi"
+        if self.gcp_enabled and self.gcp_project_id is None:
+            valid_config = False
+        if not valid_config:
+            raise BadConfigException("Invalid configuration")
